/* deux locomotives en voie unique sur le circuit imprimé du V&V
 * croisement en gare intermédiaire
 * msport 3/6/2022
 * Utilisation de la bibliothèque DCCpp sur la voie MAIN
 * Utilise plusieurs parties du programme DCCppVVOled de Dominique de Locoduino (c) Article 311
 * le V&V doit être alimenté en 12/18 V pour que le bouton SEL fonctionne, du à un partage de l'entrée A2 avec la mesure de tension
 * 
 * Version sans configuration initiale voir lignes 57 et suivantes
 * 
 * Attention pin 10/12 pour le DCC et A3 pour PIN_BtnUp sur le pcb C6 (BtnUp 12 sur le pcb C1)
 * 
 * DCCpp traite  Activate et !  Activate à l'inverse de la NMRA : ce programme y est adapté
*/

#include <DCCpp.h>
#include <EEPROM.h>

char * VERS = "V&V x2 0.1 ";  // affichage initial
char* titre[]={"V&V x2 0.1 ", "            ", "positions  ", "MARCHE     ", "lire loco", "codeur     ", 
"AUTOMATE   ", "ARRET       ", "CONFIGURE  ", "mise a quai", "attente    "}; //  11 de 0 à 5 et 6 à 10

// Include the Bounce2 library found here : https://github.com/thomasfredericks/Bounce2
#include <Bounce2.h>
Bounce debouncer1 = Bounce(); // Instantiate  Bounce objects
Bounce debouncer2 = Bounce();
Bounce debouncer3 = Bounce();
Bounce debouncer4 = Bounce();
#include <Wire.h>
#include "SSD1306Ascii.h"
#include "SSD1306AsciiWire.h"
#define I2C_ADDRESS 0x3C // 0X3C+SA0 - 0x3C or 0x3D
SSD1306AsciiWire oled;
#include <Encoder.h>              // encodeur
//ENCODER
Encoder myEnc(4,2);             
long oldPosition  = -999;         // encoder
unsigned long encoderMillis;

#define MOTOR_SHIELD_TYPE    0    // LMD18200 ou L6203 
#define COMM_INTERFACE       0
  
#define  PIN_BtnEnc         A1    // poussoir OK (Encodeur)
#define  PIN_BtnSel         A2    // poussoir SEL (Marche / Arret DCC)
#define  PIN_BtnUp          12 // A3 sur pcb C6  // poussoir UP (Manuel/Auto Automate)
#define  PIN_BtnDown        13    // poussoir DOWN (Lumière)
#define  PIN_GARE1           5     
#define  PIN_ZONEA           6    
#define  PIN_ZONEB           7    
#define  PIN_ZONEC           8    
#define  PIN_GARE2           9  
#define  LEDROUGE           11    // led rouge 
#define VOLTAGE             A2    // voltage sensor
float gtension=0;

int debug = 0; // set to 1 to show debug info on serial port 
unsigned long temps;
unsigned long temps2;
unsigned long tempsA = 5000;  // temps d'attente en gare	 
unsigned long tempsD = 5000;
unsigned long delta_t = 250;  // le pas en ms pour changer la vitesse
int i;
int j;
int adresseLoco1 = 19;
int adresseLoco2 = 68;
int etatLoco1 = 99;
int etatLoco2 = 99;
bool sens1 = 1;            // Loco1 part en avant
bool sens2 = 1;            // poser Loco2 en sens inverse
bool arretee1 = 0;
bool arretee2 = 0;

FunctionsState LocoFunctions;

bool Gare1;
bool ZoneA;
bool ZoneB;
bool ZoneC;
bool Gare2;

int vitesse1;
int vitesse2;
int vitesse_max1 = 20;
int vitesse_max2 = 20;
int vitesse_min1 = 8;
int vitesse_min2 = 8;
bool sens_1 = 1;
bool sens_2 = 0;
int Address_son =  3 ; // accessoire 9
int Address =  2 ; // accessoire 5
byte SubAddress = 0;  // accessoire 5  +1 => 6 
bool Activate = 0 ; // direct

void majOled(int numero) {  // numero du titre affiché sur l'oled
  if ( etatLoco1 == 99 ) {numero = 10;}
  oled.setCursor(0, 0); // start of 1st line
  oled.print(titre[6]);    oled.print(etatLoco1); oled.println(F(" "));   // Automate 
  oled.setCursor(0, 2); // start of 2nd line
  oled.print(titre[numero]);    oled.print(etatLoco1); oled.println(F(" "));   // 2 = positions 10 = attente
  oled.setCursor(0, 4); // start of 3rd line
  oled.print(F("Loco1 : "));   oled.print(etatLoco1); oled.println(F(" "));   
  oled.setCursor(0, 6); // start of 4th line
  oled.print(F("Loco2 : "));   oled.print(etatLoco2); oled.println(F(" "));   
}

void setup() {
  debouncer1.attach(12, INPUT_PULLUP ); // UP rouge
  debouncer1.interval(5); // interval in ms
  debouncer2.attach(13, INPUT_PULLUP ); // DOWN noir
  debouncer2.interval(5); // interval in ms
  debouncer3.attach(16, INPUT_PULLUP ); // SEL A2 vert
  debouncer3.interval(5); // interval in ms
  debouncer4.attach(15, INPUT_PULLUP ); // coder button A1
  debouncer4.interval(5); // interval in ms
  Wire.begin();         
  oled.begin(&Adafruit128x64, I2C_ADDRESS);
  oled.set400kHz();
  oled.setFont(System5x7);  
  oled.set2X();
  oled.clear();
  oled.setCursor(0, 0); // start of  
  oled.println(VERS); 
  Serial.begin( 115200 );
  DCCpp::begin(); //
  DCCpp::beginMain(UNDEFINED_PIN, 10, 3, 14); // pour voie Main 
  DCCpp::powerOn();
  for ( int i = 5; i < 10 ; i++ ) {
    pinMode(i, INPUT_PULLUP);
  }
  pinMode(LEDROUGE, OUTPUT);
  pinMode(VOLTAGE, INPUT);
  gtension = analogRead(VOLTAGE); // lecture tension alim
  gtension = (gtension *  0.030) -3 ; // 0.029;
  oled.setCursor(0,6);                        // 4eme ligne
  oled.print(F("Tension")); 
  oled.print((int)gtension); 
  oled.print(F("V")); 
  pinMode(PIN_BtnEnc, INPUT_PULLUP);    // pour lecture adresse loco1 fait aussi par Bounce2
  pinMode(PIN_BtnSel, INPUT_PULLUP);    // pour lecture adresse loco2
  delay (500); 
  DCCpp ::setAccessory(Address, SubAddress, Activate);        // 1ere aiguille directe    Addresse 5   
  delay (500);
  DCCpp ::setAccessory(Address, SubAddress + 1 , ! Activate);        // 2e aiguille déviée    Addresse 6
  delay (500);
  DCCpp ::setAccessory(Address, SubAddress, ! Activate);        // 1ere aiguille directe vers A    Addresse 5   
  delay (500);
  DCCpp ::setAccessory(Address, SubAddress + 1 , Activate);        // 2e aiguille déviée vers C   Addresse 6 
  delay (500);

  majOled(0);
 
} // END SETUP

// ============================= LOOP ==============================  
void loop() {
  DCCpp::loop(); 
  majOled(2);
  if (debug == 1){  // modif
    Serial.print("Etat Locos : ");
    Serial.print(etatLoco1);
    Serial.print(" : ");
    Serial.println(etatLoco2);
    }   
  
  debouncer3.update(); // SEL vert lancement 
  int value3 = debouncer3.read();
  if ( debouncer3.fell() ) {
    DCCpp::powerOn(); 
    oled.setCursor(0, 0); // start of 1st line demarrage
    oled.print(titre[6]);
    oled.setCursor(0, 2); // start of 2e line mise à quai
    oled.print(titre[9]); 
    etatLoco1 = -1 ;  
    etatLoco2 = -1 ;  
    delay (500);
    DCCpp ::setAccessory(Address, SubAddress, ! Activate);         // 1ere aiguille directe vers A    Addresse 5 
    delay (500);
    DCCpp ::setAccessory(Address, SubAddress + 1 , Activate);      // 2e aiguille déviée vers C       Addresse 6 
    digitalWrite(LEDROUGE, HIGH);
  } 
   
  debouncer1.update(); // UP rouge stop
  int value1 = debouncer1.read();
  if ( debouncer1.fell() ) { 
    oled.setCursor(0, 2); // start of 1st line arret 
    oled.print(titre[7]); 
    DCCpp::powerOff(); 
    digitalWrite(LEDROUGE, LOW);
    delay (2000);
  }
  
  debouncer2.update(); // DOWN noir lire adresse Loco
  int value2 = debouncer2.read();
  if ( debouncer2.fell() ) { 
    oled.setCursor(0, 0);
    oled.print(titre[4]); 
    oled.setCursor(0, 2); // start of 2rd line
    oled.print(F("           "));
    oled.setCursor(0, 2); // start of 2rd line  
    int adLoco = DCCpp::identifyLocoIdMain();
    oled.print(F("Loco # = ")); oled.print(adLoco);    
    delay (2000);
  }
  
  debouncer4.update(); // coder button  MARCHE 
  int value4 = debouncer4.read();
  if ( debouncer4.fell() ) { 
    oled.setCursor(0, 2); // start of 1st line MARCHE 
    oled.print(titre[3]); 
    digitalWrite(LEDROUGE, HIGH);
    DCCpp::powerOn(); 
    delay (2000);
  } 

// ============================= LOCO 1 ==============================  

  if ( etatLoco1 == -1 ) {    // ==== etat -1 ==== L'AUTOMATE EST LANCE 
    majOled(2);
    oled.setCursor(0, 0); // start of 1st line arret 
    oled.print(titre[6]);  // L'AUTOMATE EST DEMARRE
    if ( vitesse1 < vitesse_min1 ) {                     // on avance vers GARE1
      if (millis()> ( temps + delta_t)){
        temps = millis();
        vitesse1++ ;
        arretee1 = false ;
        DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, sens1); 
      }
    }
    else {
      etatLoco1 = 0  ; 
      DCCpp ::setAccessory(3, 0, ! Activate);         // on avance vers GARE1 vitesse constante : annonce màq train 
      delay(150);
    }     
  } 
  if ( ( etatLoco1 == 0 ) || ( etatLoco1 == 1 ) ) {    // ==== etat 0 ==== ON VA S'ARRETER GARE1
    Gare1 = ! digitalRead(PIN_GARE1);          
    if ( Gare1 ) {etatLoco1 = 1;}    //  on est arrivé Gare1
    if ( etatLoco1 == 1 ) {            // on est passé sur PIN_GARE1
      if ( vitesse1 > 0 ) {
        if (millis()> ( temps + delta_t)){
          temps = millis();
          vitesse1-- ;
          DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, sens1); 
        }
      }
      else { 
        arretee1 = true ; // ==== etat 0 ==== ON S'EST ARRETE GARE1
      }    
    }
    if ( arretee1 ) {
      DCCpp ::setAccessory(3, 0, Activate);         // on rearme annonce arrivee train en gare
      delay(150);
      DCCpp ::setAccessory(3, 1, ! Activate);         // on annonce le prochain départ
      if ( millis() < ( temps + tempsA )){ // on attend tempsA en Gare1
      }
      else {                               //  on a attendu tempsA Gare1, on repart -> ZONEA
        if ( etatLoco2 < 3 ) { 
          etatLoco1 = 2 ;
          DCCpp ::setAccessory(5, 0, ! Activate);         // on met le 1er feu au vert
          delay (100);
          DCCpp ::setAccessory(Address, SubAddress ,  ! Activate);        // 1e aiguille directe    Addresse 5
        }
        Gare1 = ! digitalRead(PIN_GARE1); // ==== fin etat 1 ==== ON S'EST ARRETE APRES GARE 1
      }
    }                          
  }  
      
// etatLoco1 = 2  
  if ( ( etatLoco1 == 2 ) || ( etatLoco1 == 3 ) ) {   // ==== etat 2 ==== GARE1 -> ZONEA    ON REPART APRES GARE 1
    majOled(2);
    oled.setCursor(0, 0); // start of 1st line arret 
    oled.print(titre[6]); 
    if ( etatLoco1 == 2 ) {                             // on n'est pas encore  sur PIN_ZONEA
      if ( vitesse1 < vitesse_max1 ) {                     // on accélère
        if (millis()> ( temps + delta_t)){
          temps = millis();
          vitesse1++ ;
          arretee1 = false ;
          DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, sens1); 
        }
      }
    }
    ZoneA = ! digitalRead(PIN_ZONEA);    
    if ( ZoneA ) {etatLoco1 = 3;}
    if ( etatLoco1 == 3 ) {                        // on est passé sur PIN_ZONEA
      DCCpp ::setAccessory(3, 2, ! Activate);         //   annonce arrivee train en gare ZONEA
      delay(150);
      DCCpp ::setAccessory(3, 1, Activate);         // on rearme :  annonce le prochain départ
      if ( vitesse1 > 0 ) {
        if (millis()> ( temps + delta_t)){
          temps = millis();
          vitesse1-- ;
          DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, sens1); 
        }
      }
      else { arretee1 = true ;}    
    }
    if ( arretee1 ) {
      if ( millis() < ( temps + tempsA )){ // on attend tempsA en ZoneA
      }
      else {                               //  tempsA écoulé, on repart -> GARE2
          if ( etatLoco2 >= 3 ) { 
            etatLoco1 = 4 ;
            DCCpp ::setAccessory(5, 0, Activate);         // on met le feu 1 au rouge
            delay(100);
            DCCpp ::setAccessory(5, 1, ! Activate);         // on met le feu 2 au vert          
            delay(100);
            DCCpp ::setAccessory(Address, SubAddress + 1 , ! Activate);       // 2e aiguille directe    Addresse 6
          } 
          ZoneA = ! digitalRead(PIN_ZONEA); // ==== etat 4 ==== ON S'EST ARRETE APRES PIN_ZONEA
      }
    } 
  }
// etatLoco1 = 4
  if  ( ( etatLoco1 == 4 ) || ( etatLoco1 == 5 ) ) {   // ==== etat 4 ==== ZONEA -> GARE2
    majOled(2);
    if ( etatLoco1 == 4 ) {                             // on n'est pas encore  sur PIN_GARE2
      if ( vitesse1 < vitesse_max1 ) {                     // on accélère
        if (millis()> ( temps + delta_t)){
          temps = millis();
          vitesse1++ ;
          arretee1 = false ;
          DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, sens1); 
        }   
      }
    }
    Gare2 = ! digitalRead(PIN_GARE2);
    if ( Gare2 ) { etatLoco1 = 5 ; }
    if ( etatLoco1 == 5 ) {                            // on est passé sur PIN_GARE2
      if ( vitesse1 > 0 ) {                            // on ralentit
        if (millis()> ( temps + delta_t)){
          temps = millis();
          vitesse1-- ;
          DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, sens1); 
        }
      }
      else { 
        arretee1 = true ; 
        DCCpp ::setAccessory(3, 3, ! Activate);         // on est GARE2 : annonce terminus generique
        delay(150);
        DCCpp ::setAccessory(3, 2, Activate);         // on rearme :  annonce arrivee train en gare ZONEA
      }   
    }
    if ( arretee1 ) {
      if ( millis() < ( temps + tempsA )){ // on attend tempsA en Gare2
      }
      else {                               //  on a attendu tempsA Gare2, on repart -> ZONEC
        if ( etatLoco2 > 4 ) { 
           etatLoco1 = 6 ;
           DCCpp ::setAccessory(Address, SubAddress + 1 , Activate);       // 2e aiguille déviée vers C   Addresse 6
           delay(100);
           DCCpp ::setAccessory(5, 1, Activate);         // on met le feu 2 au rouge
           delay(100);           
           DCCpp ::setAccessory(5, 2, ! Activate);         // on met le feu 3 au vert
           delay(150);           
           DCCpp ::setAccessory(4, 0, ! Activate);         // on est GARE2 : annonce depart 
           delay(150);
           DCCpp ::setAccessory(3, 3, Activate);         // on rearme : annonce terminus generique
        } 
      }
    }
  }
  
//  etatLoco1 = 6
  if  ( ( etatLoco1 == 6 ) || ( etatLoco1 == 7 ) ) {   // ==== etat 5 ====  retour  GARE2 -> ZONEC
    majOled(2);
    if ( etatLoco1 == 6 ) {                             // on n'est pas encore
      if ( vitesse1 < vitesse_max1 ) {                     // on accélère
        if (millis()> ( temps + delta_t)){
          temps = millis();
          vitesse1++ ;
          arretee1 = false ;
          DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, ! sens1); 
        }
      }
    }
    ZoneC = ! digitalRead(PIN_ZONEC);
    if ( ZoneC ) { etatLoco1 = 7; }
    if ( etatLoco1 == 7 ) {                      // on est passé sur PIN_ZONEC et on s'arrête
      if ( vitesse1 > 0 ) {
        if (millis()> ( temps + delta_t)){
          temps = millis();
          vitesse1-- ;
          DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, ! sens1); 
        }
      }
      else { 
        arretee1 = true ;
        DCCpp ::setAccessory(4, 0, Activate);         // on rearme :  GARE2 : annonce depart 
        delay(150);       
        DCCpp ::setAccessory(4, 1, ! Activate);         // on est ZoneC : annonce itineraire 
      }    
    }
    if ( arretee1 ) {
      if ( millis() < ( temps + tempsA )){ // on attend tempsA
      }
      else {                               // on a attendu tempsA ZoneC, on repart -> GARE1
        if ( etatLoco2 > 6 ) { 
          etatLoco1 = 8 ;
          DCCpp ::setAccessory(Address, SubAddress, Activate);        // 1e aiguille déviée vers C      Addresse 5
          delay(100);
          DCCpp ::setAccessory(5, 2, Activate);         // on met le feu 3 au rouge
          delay(100);
          DCCpp ::setAccessory(5, 3, ! Activate);         // on met le feu 4 au vert
          delay(150);
          DCCpp ::setAccessory(4, 1, Activate);         // on rearme : ZoneC : annonce itineraire
          delay(150);
          DCCpp ::setAccessory(4, 2, ! Activate);         // on est ZoneC : depart 
        }
        ZoneC = ! digitalRead(PIN_ZONEC);
      }
    }          
  }


// etatLoco1 = 8
  if  ( ( etatLoco1 == 8 ) || ( etatLoco1 == 9 ) ) {   // ==== etat 8 ==== ZONEC -> GARE1
    majOled(2);
    if ( etatLoco1 == 8 ) {  
        Gare1 = ! digitalRead(PIN_GARE1) ; // ==== etat 8 ==== ON S'EST ARRETE APRES PIN_ZONEC ON REPART
        if ( vitesse1 < vitesse_max1 ) {
          if (millis()> ( temps + delta_t)){
            temps = millis();
            vitesse1++ ;
            arretee1 = false ;
            DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, ! sens1); 
          }   
       }
    }
    Gare1 = ! digitalRead(PIN_GARE1);// ==== etat 9 ==== ON EST ARRIVE PIN_GARE1 ON S'ARRETE
    if ( Gare1 ){etatLoco1 = 9 ;}          //  on est arrivé Gare1
    if ( etatLoco1 == 9 ) {      
      if ( vitesse1 > 0 ) {
        if (millis()> ( temps + delta_t)){
          temps = millis();
          vitesse1-- ;
          DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, ! sens1); 
        }
      }
      else {
        arretee1 = true ;
        DCCpp ::setAccessory(4, 3, ! Activate);         // on est GARE1 : annonce terminus 
        delay(150);
        DCCpp ::setAccessory(4, 2, Activate);         // on rearme :  ZoneC : depart 
      }    
    }
    if ( arretee1 ) {
      if ( millis() < ( temps + tempsA )){ // on attend tempsA en Gare1
      }
      else {                               // on a attendu tempsA Gare1, on repart
        etatLoco1 = 10 ;
        DCCpp ::setAccessory(5, 3, Activate);         // on met le feu 2 au rouge
        delay(100);
        DCCpp ::setAccessory(5, 0, ! Activate);         // on met le feu 1 au vert
      }
    }           
  }
  
  if  ( etatLoco1 == 10 )  {   // ==== etat 10 ====
    majOled(2);
    Gare1 = ! digitalRead(PIN_GARE1) ;  // ==== etat 10 ==== ON S'EST ARRETE APRES GARE1
    if ( vitesse1 < vitesse_min2 ) {
      if (millis()> ( temps + delta_t)){
        temps = millis();
        vitesse1++ ;
        arretee1 = false ;
        DCCpp::setSpeedMain(1, adresseLoco1, 128, vitesse1, sens1); // ==== etat 10 ==== ON REPART VERS GARE1 EN SENS NORMAL A PETITE VITESSE
        DCCpp ::setAccessory(4, 3, Activate);         // on rearme :  annonce terminus  
        delay(150);
      }  
    }
    else  {
      etatLoco1 = -1;                          // on repart pour un cycle        
    }
  } 
 // END LOCO 1

  
// ============================= LOCO 2 ==============================  
    
// etatLoco2 = 0
  if ( etatLoco2 == -1 ) {    // ==== etat -1 ==== ON LANCE L'AUTOMATE
    majOled(2);
    oled.setCursor(0, 0); // start of 1st line arret 
    oled.print(titre[6]); 
    if ( vitesse2 < vitesse_min2 ) {                     // on avance vers GARE2
      if (millis()> ( temps2 + delta_t)){
        temps2 = millis();
        vitesse2++ ;
        arretee2 = false ;
        DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, sens2); 
      }
    }
    else { etatLoco2 = 0  ;  }
  } 
  if ( ( etatLoco2 == 0 ) || ( etatLoco2 == 1 ) ) {    // ==== etat 0 ==== ON VA S'ARRETER GARE2
    Gare2 = ! digitalRead(PIN_GARE2);          
    if ( Gare2 ) {etatLoco2 = 1;}    //  on est arrivé Gare1
    if ( etatLoco2 == 1 ) {            // on est passé sur PIN_GARE2
      if ( vitesse2 > 0 ) {
        if (millis()> ( temps2 + delta_t)){
          temps2 = millis();
          vitesse2-- ;
          DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, sens2); 
        }
      }
      else { arretee2 = true ;}    
    }
    if ( arretee2 ) {
      if ( millis() < ( temps2 + tempsA )){ // on attend tempsA Gare2
      }
      else {                               //  tempsA écoulé, on repart -> ZONEC 
        if ( etatLoco1 < 3 ) { 
          etatLoco2 = 2 ;
          DCCpp ::setAccessory(Address, SubAddress +1, Activate);        // 2e aiguille déviée vers C   Addresse 6
          delay(100);
          DCCpp ::setAccessory(5, 2, ! Activate);         // on met le feu 3 au vert
        }
        Gare2 = ! digitalRead(PIN_GARE2); // ==== fin etat 1 ====  ON S ARRETE APRES GARE 2
      }
    }                          
  }  
      
// etatLoco2 = 2  
  if ( ( etatLoco2 == 2 ) || ( etatLoco2 == 3 ) ) {   // ==== etat 2 ==== GARE2 -> ZONEC   
    majOled(2);
    oled.setCursor(0, 0); // start of 1st line arret 
    oled.print(titre[6]); 
    if ( etatLoco2 == 2 ) {                             // on n'est pas encore  sur PIN_ZONEC
      if ( vitesse2 < vitesse_max2 ) {                     // on accélère
        if (millis()> ( temps2 + delta_t)){
          temps2 = millis();
          vitesse2++ ;
          arretee2 = false ;
          DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, sens2); 
        }
      }
    }
    ZoneC = ! digitalRead(PIN_ZONEC);    
    if ( ZoneC ) {etatLoco2 = 3;}
    if ( etatLoco2 == 3 ) {                        // on est passé sur PIN_ZONEC
      if ( vitesse2 > 0 ) {
        if (millis()> ( temps2 + delta_t)){
          temps2 = millis();
          vitesse2-- ;
          DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, sens2); 
        }
      }
      else { arretee2 = true ;}    
    }
    if ( arretee2 ) {
      if ( millis() < ( temps2 + tempsA )){ // on attend tempsA
      }
      else {                               //  on attendu tempsA ZONEC, on repart vers Gare1
          if ( etatLoco1 >= 3 ) { 
            etatLoco2 = 4 ;
            DCCpp ::setAccessory(Address, SubAddress, Activate);        // 1ere aiguille déviée vers C   Addresse 5
            delay(100);
            DCCpp ::setAccessory(5, 2, Activate);         // on met le feu 3 au rouge
            delay(100);
            DCCpp ::setAccessory(5, 3, ! Activate);         // on met le feu 4 au vert
          } 
          ZoneC = ! digitalRead(PIN_ZONEC); // ==== etat 4 ==== ON S'EST ARRETE APRES PIN_ZONEC
      }
    } 
  }
  
// etatLoco2 = 4
  if  ( ( etatLoco2 == 4 ) || ( etatLoco2 == 5 ) ) {   // ==== etat 4 ==== ZONEC -> GARE1
    majOled(2);
    if ( etatLoco2 == 4 ) {                             // on n'est pas encore  sur PIN_GARE1
      if ( vitesse2 < vitesse_max2 ) {                     // on accélère
        if (millis()> ( temps2 + delta_t)){
          temps2 = millis();
          vitesse2++ ;
          arretee2 = false ;
          DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, sens2); 
        }   
      }
    }
    Gare1 = ! digitalRead(PIN_GARE1);
    if ( Gare1 ) { etatLoco2 = 5 ; }
    if ( etatLoco2 == 5 ) {                            // on est passé sur PIN_GARE1
      if ( vitesse2 > 0 ) {                            // on ralentit
        if (millis()> ( temps2 + delta_t)){
          temps2 = millis();
          vitesse2-- ;
          DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, sens2); 
        }
      }
      else { arretee2 = true ;}    
    }
    if ( arretee2 ) {
      if ( millis() < ( temps2 + tempsA )){ // on attend tempsA Gare1
      }
      else {                               //  on a attendu tempsA Gare1, on repart vers  ZoneA
        if ( etatLoco1 > 4 ) { 
           etatLoco2 = 6 ;
           DCCpp ::setAccessory(Address, SubAddress, ! Activate);       // 1ere aiguille directe vers A  Addresse 5
           delay(100);
           DCCpp ::setAccessory(5, 3, Activate);         // on met le feu 4 au rouge
           delay(100);
           DCCpp ::setAccessory(5, 0, ! Activate);         // on met le feu 4 au vert
        } 
        Gare1 = ! digitalRead(PIN_GARE1);
      }
    }
  }
  
// etatLoco2 = 6
  if  ( ( etatLoco2 == 6 ) || ( etatLoco2 == 7 ) ) {   // ==== etat 5 ====  retour  GARE1 -> ZONEA
    majOled(2);
    if ( etatLoco2 == 6 ) {                             // on n'est pas encore
      if ( vitesse2 < vitesse_max2 ) {                     // on accélère
        if (millis()> ( temps2 + delta_t)){
          temps2 = millis();
          vitesse2++ ;
          arretee2 = false ;
          DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, ! sens2); 
        }
      }
    }
    ZoneA = ! digitalRead(PIN_ZONEA);
    if ( ZoneA ) { etatLoco2 = 7; }
    if ( etatLoco2 == 7 ) {                      // on est passé sur PIN_ZONEA et on s'arrête
      if ( vitesse2 > 0 ) {
        if (millis()> ( temps2 + delta_t)){
          temps2 = millis();
          vitesse2-- ;
          DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, ! sens2); 
        }
      }
      else { arretee2 = true ;}    
    }
    if ( arretee2 ) {
      if ( millis() < ( temps2 + tempsA )){ // on attend tempsA
      }
      else {                               //  on a attendu tempsA ZoneA, on repart -> GARE2
        if ( etatLoco1 > 6 ) {
          etatLoco2 = 8 ;
          DCCpp ::setAccessory(5, 0, Activate);         // on met le feu 1 au rouge
          delay(100);
          DCCpp ::setAccessory(5, 1, ! Activate);         // on met le feu 2 au vert
          delay(100);
          DCCpp ::setAccessory(Address, SubAddress + 1, ! Activate);        // 2e aiguille directe vers A   Addresse 6
        }           
          
      }
        ZoneA = ! digitalRead(PIN_ZONEA);
    }
  }          



// etatLoco2 = 8
  if  ( ( etatLoco2 == 8 ) || ( etatLoco2 == 9 ) ) {   // ==== etat 8 ==== ZONEA -> GARE2
    majOled(2);
    if ( etatLoco2 == 8 ) {  
        Gare2 = ! digitalRead(PIN_GARE2) ; // ==== etat 8 ==== ON S'EST ARRETE APRES PIN_ZONEA ON REPART
        if ( vitesse2 < vitesse_max2 ) {
          if (millis()> ( temps2 + delta_t)){
            temps2 = millis();
            vitesse2++ ;
            arretee2 = false ;
            DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, ! sens2); 
          }   
       }
    }
    Gare2 = ! digitalRead(PIN_GARE2);// ==== etat 9 ==== ON EST ARRIVE GARE2 ON S'ARRETE
    if ( Gare2 ){etatLoco2 = 9 ;}          //  on est arrivé Gare2
    if ( etatLoco2 == 9 ) {      
      if ( vitesse2 > 0 ) {
        if (millis()> ( temps2 + delta_t)){
          temps2 = millis();
          vitesse2-- ;
          DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, ! sens2); 
        }
      }
      else { arretee2 = true ;}    
    }
    if ( arretee2 ) {
      if ( millis() < ( temps2 + tempsA )){ // on attend tempsA
      }
      else {                               //  on a attendu tempsA Gare2, on repart
          etatLoco2 = 10 ;
          DCCpp ::setAccessory(5, 1, Activate);         // on met le feu 2 au rouge
          delay(100);
          DCCpp ::setAccessory(5, 2, ! Activate);         // on met le feu 3 au vert
      }
    }           
  }
  
  if  ( etatLoco2 == 10 )  {   // ==== etat 10 ====
    majOled(2);
    Gare2 = ! digitalRead(PIN_GARE2) ;  // ==== etat 10 ==== ON S'EST ARRETE APRES GARE2
    if ( vitesse2 < vitesse_min2 ) {
      if (millis()> ( temps2 + delta_t)){
        temps2 = millis();
        vitesse2++ ;
        arretee2 = false ;
        DCCpp::setSpeedMain(1, adresseLoco2, 128, vitesse2, sens2); // ==== etat 10 ==== ON REPART VERS GARE2 EN SENS OPPOSE A PETITE VITESSE
      }  
    }
    else  {
      etatLoco2 = -1;                          // on repart pour un cycle        
    }
  } 
 // END LOCO 2 
 
} // END LOOP
